'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.writeFile = exports.createWriteStream = exports.DOMImage = exports.ImageData = exports.createCanvas = exports.env = undefined;
exports.fetchBinary = fetchBinary;

var _fs = require('fs');

var message = 'requires the canvas library. Install it with `npm install canvas@next`.';

// eslint-disable-next-line import/no-mutable-exports
var createCanvas = void 0,
    DOMImage = void 0,
    ImageData = void 0;
try {
  var canvas = require('canvas');
  exports.createCanvas = createCanvas = canvas.createCanvas;
  exports.DOMImage = DOMImage = canvas.Image;
  exports.ImageData = ImageData = canvas.ImageData;
} catch (e) {
  exports.createCanvas = createCanvas = function createCanvas() {
    throw new Error(`createCanvas ${message}`);
  };
  exports.DOMImage = DOMImage = function DOMImage() {
    throw new Error(`DOMImage ${message}`);
  };
  exports.ImageData = ImageData = function ImageData() {
    throw new Error(`ImageData ${message}`);
  };
}

var env = exports.env = 'node';

function fetchBinary(path) {
  return new Promise(function (resolve, reject) {
    (0, _fs.readFile)(path, function (err, data) {
      if (err) reject(err);else resolve(data.buffer);
    });
  });
}

exports.createCanvas = createCanvas;
exports.ImageData = ImageData;
exports.DOMImage = DOMImage;
exports.createWriteStream = _fs.createWriteStream;
exports.writeFile = _fs.writeFile;