'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = blurFilter;
/**
 * Blurs the image by averaging the neighboring pixels.
 * @memberof Image
 * @instance
 * @param {object} [options]
 * @param {number} [options.radius=1] - Number of pixels around the current pixel to average
 * @return {Image}
 */
function blurFilter(options = {}) {
  var _options$radius = options.radius,
      radius = _options$radius === undefined ? 1 : _options$radius;


  if (radius < 1) {
    throw new Error('radius must be greater than 1');
  }

  var n = 2 * radius + 1;
  var kernel = new Array(n);
  for (var i = 0; i < n; i++) {
    kernel[i] = new Array(n);
    for (var j = 0; j < n; j++) {
      kernel[i][j] = 1;
    }
  }

  for (var _i = 0; _i < kernel.length; _i++) {
    kernel[_i] = 1;
  }

  return this.convolution(kernel);
}