'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getThreshold;

var _thresholdAlgorithms = require('../transform/mask/thresholdAlgorithms');

/**
 * Returns a threshold for the creation of a binary mask with the `mask()` method.
 * @memberof Image
 * @instance
 * @param {object} [options]
 * @param {ThresholdAlgorithm} [options.algorithm='otsu']
 * @return {number}
 */
function getThreshold(options = {}) {
  var _options$algorithm = options.algorithm,
      algorithm = _options$algorithm === undefined ? _thresholdAlgorithms.names.otsu : _options$algorithm;


  this.checkProcessable('getThreshold', {
    components: 1,
    bitDepth: [8, 16]
  });

  var method = _thresholdAlgorithms.methods[algorithm.toLowerCase()];
  if (method) {
    var histogram = this.getHistogram();
    return method(histogram, this.size);
  } else {
    throw new Error(`unknown thresholding algorithm: ${algorithm}`);
  }
}